@@ -66,9 +66,23 @@
 //!
 //! [See example here](https://github.com/libbpf/libbpf-rs/tree/master/examples/runqslower).
 
+#![allow(clippy::let_unit_value)]
+#![warn(
+    elided_lifetimes_in_paths,
+    missing_debug_implementations,
+    missing_docs,
+    single_use_lifetimes,
+    clippy::absolute_paths,
+    clippy::wildcard_imports,
+    rustdoc::broken_intra_doc_links
+)]
+#![deny(unsafe_op_in_unsafe_fn)]
+
+pub mod btf;
 mod error;
 mod iter;
 mod link;
+mod linker;
 mod map;
 mod object;
 mod perf_buffer;
@@ -76,20 +90,71 @@ mod print;
 mod program;
 pub mod query;
 mod ringbuf;
-/// Used for skeleton -- an end user may not consider this API stable
-#[doc(hidden)]
-pub mod skeleton;
+mod skeleton;
+mod tc;
 mod util;
+mod xdp;
 
 pub use libbpf_sys;
 
-pub use crate::error::{Error, Result};
+pub use crate::btf::Btf;
+pub use crate::btf::HasSize;
+pub use crate::btf::ReferencesType;
+pub use crate::error::Error;
+pub use crate::error::ErrorExt;
+pub use crate::error::ErrorKind;
+pub use crate::error::Result;
 pub use crate::iter::Iter;
 pub use crate::link::Link;
-pub use crate::map::{Map, MapFlags, MapType, OpenMap};
-pub use crate::object::{Object, ObjectBuilder, OpenObject};
-pub use crate::perf_buffer::{PerfBuffer, PerfBufferBuilder};
-pub use crate::print::{get_print, set_print, PrintCallback, PrintLevel};
-pub use crate::program::{OpenProgram, Program, ProgramAttachType, ProgramType};
-pub use crate::ringbuf::{RingBuffer, RingBufferBuilder};
+pub use crate::linker::Linker;
+pub use crate::map::Map;
+pub use crate::map::MapFlags;
+pub use crate::map::MapHandle;
+pub use crate::map::MapInfo;
+pub use crate::map::MapType;
+pub use crate::map::OpenMap;
+pub use crate::object::AsRawLibbpf;
+pub use crate::object::Object;
+pub use crate::object::ObjectBuilder;
+pub use crate::object::OpenObject;
+pub use crate::perf_buffer::PerfBuffer;
+pub use crate::perf_buffer::PerfBufferBuilder;
+pub use crate::print::get_print;
+pub use crate::print::set_print;
+pub use crate::print::PrintCallback;
+pub use crate::print::PrintLevel;
+pub use crate::program::OpenProgram;
+pub use crate::program::Program;
+pub use crate::program::ProgramAttachType;
+pub use crate::program::ProgramType;
+pub use crate::program::TracepointOpts;
+pub use crate::program::UprobeOpts;
+pub use crate::program::UsdtOpts;
+pub use crate::ringbuf::RingBuffer;
+pub use crate::ringbuf::RingBufferBuilder;
+pub use crate::tc::TcAttachPoint;
+pub use crate::tc::TcHook;
+pub use crate::tc::TcHookBuilder;
+pub use crate::tc::TC_CUSTOM;
+pub use crate::tc::TC_EGRESS;
+pub use crate::tc::TC_H_CLSACT;
+pub use crate::tc::TC_H_INGRESS;
+pub use crate::tc::TC_H_MIN_EGRESS;
+pub use crate::tc::TC_H_MIN_INGRESS;
+pub use crate::tc::TC_INGRESS;
 pub use crate::util::num_possible_cpus;
+pub use crate::xdp::Xdp;
+pub use crate::xdp::XdpFlags;
+
+/// Used for skeleton -- an end user may not consider this API stable
+#[doc(hidden)]
+pub mod __internal_skel {
+    pub use super::skeleton::*;
+}
+
+/// Skeleton related definitions.
+pub mod skel {
+    pub use super::skeleton::OpenSkel;
+    pub use super::skeleton::Skel;
+    pub use super::skeleton::SkelBuilder;
+}
