@@ -1,31 +1,98 @@
 use core::ffi::c_void;
 use std::collections::HashMap;
 use std::ffi::CStr;
+use std::ffi::CString;
 use std::mem;
-use std::os::raw::c_char;
 use std::path::Path;
 use std::ptr;
+use std::ptr::NonNull;
 
+use crate::libbpf_sys;
+use crate::set_print;
 use crate::util;
-use crate::*;
+use crate::Btf;
+use crate::Error;
+use crate::Map;
+use crate::OpenMap;
+use crate::OpenProgram;
+use crate::PrintLevel;
+use crate::Program;
+use crate::Result;
+
+/// A trait implemented for types that are thin wrappers around `libbpf` types.
+///
+/// The trait provides access to the underlying `libbpf` (or `libbpf-sys`)
+/// object. In many cases, this enables direct usage of `libbpf-sys`
+/// functionality when higher-level bindings are not yet provided by this crate.
+pub trait AsRawLibbpf {
+    /// The underlying `libbpf` type.
+    type LibbpfType;
+
+    /// Retrieve the underlying `libbpf` object.
+    ///
+    /// # Warning
+    /// By virtue of working with a mutable raw pointer this method effectively
+    /// circumvents mutability and liveness checks. While by-design, usage is
+    /// meant as an escape-hatch more than anything else. If you find yourself
+    /// making use of it, please consider discussing your workflow with crate
+    /// maintainers to see if it would make sense to provide safer wrappers.
+    fn as_libbpf_object(&self) -> NonNull<Self::LibbpfType>;
+}
 
 /// Builder for creating an [`OpenObject`]. Typically the entry point into libbpf-rs.
-#[derive(Default)]
+#[derive(Debug)]
 pub struct ObjectBuilder {
-    name: String,
-    relaxed_maps: bool,
+    name: Option<CString>,
+    pin_root_path: Option<CString>,
+
+    opts: libbpf_sys::bpf_object_open_opts,
+}
+
+impl Default for ObjectBuilder {
+    fn default() -> Self {
+        let opts = libbpf_sys::bpf_object_open_opts {
+            sz: mem::size_of::<libbpf_sys::bpf_object_open_opts>() as libbpf_sys::size_t,
+            object_name: ptr::null(),
+            relaxed_maps: false,
+            pin_root_path: ptr::null(),
+            kconfig: ptr::null(),
+            btf_custom_path: ptr::null(),
+            kernel_log_buf: ptr::null_mut(),
+            kernel_log_size: 0,
+            kernel_log_level: 0,
+            ..Default::default()
+        };
+        Self {
+            name: None,
+            pin_root_path: None,
+            opts,
+        }
+    }
 }
 
 impl ObjectBuilder {
     /// Override the generated name that would have been inferred from the constructor.
-    pub fn name<T: AsRef<str>>(&mut self, name: T) -> &mut Self {
-        self.name = name.as_ref().to_string();
-        self
+    pub fn name<T: AsRef<str>>(&mut self, name: T) -> Result<&mut Self> {
+        self.name = Some(util::str_to_cstring(name.as_ref())?);
+        self.opts.object_name = self.name.as_ref().map_or(ptr::null(), |p| p.as_ptr());
+        Ok(self)
+    }
+
+    /// Set the pin_root_path for maps that are pinned by name.
+    ///
+    /// By default, this is NULL which bpf translates to /sys/fs/bpf
+    pub fn pin_root_path<T: AsRef<Path>>(&mut self, path: T) -> Result<&mut Self> {
+        self.pin_root_path = Some(util::path_to_cstring(path)?);
+        self.opts.pin_root_path = self
+            .pin_root_path
+            .as_ref()
+            .map_or(ptr::null(), |p| p.as_ptr());
+        Ok(self)
     }
 
     /// Option to parse map definitions non-strictly, allowing extra attributes/data
     pub fn relaxed_maps(&mut self, relaxed_maps: bool) -> &mut Self {
-        self.relaxed_maps = relaxed_maps;
+        self.opts.relaxed_maps = relaxed_maps;
         self
     }
 
@@ -35,99 +102,68 @@ impl ObjectBuilder {
     /// currently in use.
     pub fn debug(&mut self, dbg: bool) -> &mut Self {
         if dbg {
-            set_print(Some((PrintLevel::Debug, |_, s| print!("{}", s))));
+            set_print(Some((PrintLevel::Debug, |_, s| print!("{s}"))));
         } else {
             set_print(None);
         }
         self
     }
 
-    /// Used for skeleton -- an end user may not consider this API stable
-    #[doc(hidden)]
-    pub fn opts(&mut self, name: *const c_char) -> libbpf_sys::bpf_object_open_opts {
-        libbpf_sys::bpf_object_open_opts {
-            sz: mem::size_of::<libbpf_sys::bpf_object_open_opts>() as libbpf_sys::size_t,
-            object_name: name,
-            relaxed_maps: self.relaxed_maps,
-            relaxed_core_relocs: false,
-            pin_root_path: ptr::null(),
-            attach_prog_fd: 0,
-            kconfig: ptr::null(),
-            btf_custom_path: ptr::null(),
-            __bindgen_padding_0: <[u8; 6]>::default(),
-            __bindgen_padding_1: <[u8; 4]>::default(),
-        }
+    /// Get the raw libbpf_sys::bpf_object_open_opts.
+    ///
+    /// The internal pointers are tied to the lifetime of the
+    /// ObjectBuilder, so be wary when copying the struct or otherwise
+    /// handing the lifetime over to C.
+    pub fn opts(&self) -> &libbpf_sys::bpf_object_open_opts {
+        &self.opts
     }
 
+    /// Open an object using the provided path on the file system.
     pub fn open_file<P: AsRef<Path>>(&mut self, path: P) -> Result<OpenObject> {
-        // Convert path to a C style pointer
-        let path_str = path.as_ref().to_str().ok_or_else(|| {
-            Error::InvalidInput(format!("{} is not valid unicode", path.as_ref().display()))
-        })?;
-        let path_c = util::str_to_cstring(path_str)?;
+        let path_c = util::path_to_cstring(path)?;
         let path_ptr = path_c.as_ptr();
 
-        // Convert name to a C style pointer
-        //
-        // NB: we must hold onto a CString otherwise our pointer dangles
-        let name = util::str_to_cstring(&self.name)?;
-        let name_ptr = if !self.name.is_empty() {
-            name.as_ptr()
-        } else {
-            ptr::null()
-        };
+        let opts = self.opts();
 
-        let opts = self.opts(name_ptr);
-
-        let obj = unsafe { libbpf_sys::bpf_object__open_file(path_ptr, &opts) };
-        let err = unsafe { libbpf_sys::libbpf_get_error(obj as *const _) };
-        if err != 0 {
-            return Err(Error::System(err as i32));
-        }
-
-        OpenObject::new(obj)
+        util::create_bpf_entity_checked(|| unsafe {
+            libbpf_sys::bpf_object__open_file(path_ptr, opts)
+        })
+        .and_then(|ptr| unsafe { OpenObject::new(ptr) })
     }
 
-    pub fn open_memory<T: AsRef<str>>(&mut self, name: T, mem: &[u8]) -> Result<OpenObject> {
-        // Convert name to a C style pointer
-        //
-        // NB: we must hold onto a CString otherwise our pointer dangles
-        let name = util::str_to_cstring(name.as_ref())?;
-        let name_ptr = if !name.to_bytes().is_empty() {
-            name.as_ptr()
-        } else {
-            ptr::null()
-        };
+    /// Open an object from memory.
+    pub fn open_memory(&mut self, mem: &[u8]) -> Result<OpenObject> {
+        let opts = self.opts();
 
-        let opts = self.opts(name_ptr);
-
-        let obj = unsafe {
+        util::create_bpf_entity_checked(|| unsafe {
             libbpf_sys::bpf_object__open_mem(
                 mem.as_ptr() as *const c_void,
                 mem.len() as libbpf_sys::size_t,
-                &opts,
+                opts,
             )
-        };
-        let err = unsafe { libbpf_sys::libbpf_get_error(obj as *const _) };
-        if err != 0 {
-            return Err(Error::System(err as i32));
-        }
-
-        OpenObject::new(obj)
+        })
+        .and_then(|ptr| unsafe { OpenObject::new(ptr) })
     }
 }
 
 /// Represents an opened (but not yet loaded) BPF object file.
 ///
 /// Use this object to access [`OpenMap`]s and [`OpenProgram`]s.
+#[derive(Debug)]
 pub struct OpenObject {
-    ptr: *mut libbpf_sys::bpf_object,
+    ptr: NonNull<libbpf_sys::bpf_object>,
     maps: HashMap<String, OpenMap>,
     progs: HashMap<String, OpenProgram>,
 }
 
 impl OpenObject {
-    fn new(ptr: *mut libbpf_sys::bpf_object) -> Result<Self> {
+    /// Create a [`OpenObject`] from a [`libbpf_sys::bpf_object`].
+    ///
+    /// # Safety
+    /// `ptr` must point to an opened but not loaded `bpf_object`
+    ///
+    /// It is not safe to manipulate `ptr` after this operation.
+    unsafe fn new(ptr: NonNull<libbpf_sys::bpf_object>) -> Result<Self> {
         let mut obj = OpenObject {
             ptr,
             maps: HashMap::new(),
@@ -135,47 +171,42 @@ impl OpenObject {
         };
 
         // Populate obj.maps
-        let mut map: *mut libbpf_sys::bpf_map = std::ptr::null_mut();
+        let mut map: *mut libbpf_sys::bpf_map = ptr::null_mut();
         loop {
             // Get the pointer to the next BPF map
-            let next_ptr = unsafe { libbpf_sys::bpf_object__next_map(obj.ptr, map) };
-            if next_ptr.is_null() {
-                break;
-            }
+            let map_ptr = {
+                let next_ptr = unsafe { libbpf_sys::bpf_object__next_map(obj.ptr.as_ptr(), map) };
+                match NonNull::new(next_ptr) {
+                    Some(map_ptr) => map_ptr,
+                    None => break,
+                }
+            };
 
-            // Get the map name
-            // bpf_map__name can return null but only if it's passed a null.
-            // We already know next_ptr is not null.
-            let name = unsafe { libbpf_sys::bpf_map__name(next_ptr) };
-            let name = util::c_ptr_to_string(name)?;
+            let map_obj = unsafe { OpenMap::new(map_ptr) };
 
             // Add the map to the hashmap
-            obj.maps.insert(name, OpenMap::new(next_ptr));
-            map = next_ptr;
+            obj.maps.insert(map_obj.name()?.into(), map_obj);
+            map = map_ptr.as_ptr();
         }
 
         // Populate obj.progs
-        let mut prog: *mut libbpf_sys::bpf_program = std::ptr::null_mut();
+        let mut prog: *mut libbpf_sys::bpf_program = ptr::null_mut();
         loop {
             // Get the pointer to the next BPF program
-            let next_ptr = unsafe { libbpf_sys::bpf_object__next_program(obj.ptr, prog) };
-            if next_ptr.is_null() {
-                break;
-            }
+            let prog_ptr = {
+                let next_ptr =
+                    unsafe { libbpf_sys::bpf_object__next_program(obj.ptr.as_ptr(), prog) };
+                match NonNull::new(next_ptr) {
+                    Some(ptr) => ptr,
+                    None => break,
+                }
+            };
 
-            // Get the program name.
-            // bpf_program__name never returns NULL, so no need to check the pointer.
-            let name = unsafe { libbpf_sys::bpf_program__name(next_ptr) };
-            let name = util::c_ptr_to_string(name)?;
-
-            // Get the program section
-            // bpf_program__section_name never returns NULL, so no need to check the pointer.
-            let section = unsafe { libbpf_sys::bpf_program__section_name(next_ptr) };
-            let section = util::c_ptr_to_string(section)?;
+            let program = unsafe { OpenProgram::new(prog_ptr) }?;
 
             // Add the program to the hashmap
-            obj.progs.insert(name, OpenProgram::new(next_ptr, section));
-            prog = next_ptr;
+            obj.progs.insert(program.name()?.into(), program);
+            prog = prog_ptr.as_ptr();
         }
 
         Ok(obj)
@@ -185,32 +216,44 @@ impl OpenObject {
     ///
     /// # Safety
     ///
-    /// If `ptr` is unopen or already loaded then further operations on the returned object are
-    /// undefined.
+    /// Operations on the returned object are undefined if `ptr` is any one of:
+    ///     - null
+    ///     - points to an unopened `bpf_object`
+    ///     - points to a loaded `bpf_object`
     ///
     /// It is not safe to manipulate `ptr` after this operation.
-    pub unsafe fn from_ptr(ptr: *mut libbpf_sys::bpf_object) -> Result<Self> {
-        Self::new(ptr)
+    pub unsafe fn from_ptr(ptr: NonNull<libbpf_sys::bpf_object>) -> Result<Self> {
+        unsafe { Self::new(ptr) }
     }
 
     /// Takes underlying `libbpf_sys::bpf_object` pointer.
-    pub fn take_ptr(mut self) -> *mut libbpf_sys::bpf_object {
-        let ptr = self.ptr;
-        self.ptr = ptr::null_mut();
+    pub fn take_ptr(mut self) -> NonNull<libbpf_sys::bpf_object> {
+        let ptr = {
+            // manually free the internal state.
+            // using destructuring we make sure we'll get a compiler error if anything in
+            // Self changes, which will alert us to change this function as well
+            let Self { ptr, maps, progs } = &mut self;
+            mem::take(maps);
+            mem::take(progs);
+            *ptr
+        };
+        // avoid double free of self.ptr
+        mem::forget(self);
         ptr
     }
 
+    /// Retrieve the object's name.
     pub fn name(&self) -> Result<&str> {
         unsafe {
-            let ptr = libbpf_sys::bpf_object__name(self.ptr);
+            let ptr = libbpf_sys::bpf_object__name(self.ptr.as_ptr());
             let err = libbpf_sys::libbpf_get_error(ptr as *const _);
             if err != 0 {
-                return Err(Error::System(err as i32));
+                return Err(Error::from_raw_os_error(err as i32));
             }
 
             CStr::from_ptr(ptr)
                 .to_str()
-                .map_err(|e| Error::Internal(e.to_string()))
+                .map_err(Error::with_invalid_data)
         }
     }
 
@@ -259,17 +302,11 @@ impl OpenObject {
     }
 
     /// Load the maps and programs contained in this BPF object into the system.
-    pub fn load(mut self) -> Result<Object> {
-        let ret = unsafe { libbpf_sys::bpf_object__load(self.ptr) };
-        if ret != 0 {
-            // bpf_object__load() returns errno as negative, so flip
-            return Err(Error::System(-ret));
-        }
+    pub fn load(self) -> Result<Object> {
+        let ret = unsafe { libbpf_sys::bpf_object__load(self.ptr.as_ptr()) };
+        let () = util::parse_ret(ret)?;
 
-        let obj = Object::new(self.ptr)?;
-
-        // Prevent object from being closed once `self` is dropped
-        self.ptr = ptr::null_mut();
+        let obj = unsafe { Object::from_ptr(self.take_ptr())? };
 
         Ok(obj)
     }
@@ -279,7 +316,7 @@ impl Drop for OpenObject {
     fn drop(&mut self) {
         // `self.ptr` may be null if `load()` was called. This is ok: libbpf noops
         unsafe {
-            libbpf_sys::bpf_object__close(self.ptr);
+            libbpf_sys::bpf_object__close(self.ptr.as_ptr());
         }
     }
 }
@@ -293,82 +330,14 @@ impl Drop for OpenObject {
 ///
 /// Note that this is an explanation of the motivation -- Rust's lifetime system should already be
 /// enforcing this invariant.
+#[derive(Debug)]
 pub struct Object {
-    ptr: *mut libbpf_sys::bpf_object,
+    ptr: NonNull<libbpf_sys::bpf_object>,
     maps: HashMap<String, Map>,
     progs: HashMap<String, Program>,
 }
 
 impl Object {
-    fn new(ptr: *mut libbpf_sys::bpf_object) -> Result<Self> {
-        let mut obj = Object {
-            ptr,
-            maps: HashMap::new(),
-            progs: HashMap::new(),
-        };
-
-        // Populate obj.maps
-        let mut map: *mut libbpf_sys::bpf_map = std::ptr::null_mut();
-        loop {
-            // Get the pointer to the next BPF map
-            let next_ptr = unsafe { libbpf_sys::bpf_object__next_map(obj.ptr, map) };
-            if next_ptr.is_null() {
-                break;
-            }
-
-            // Get the map name
-            // bpf_map__name can return null but only if it's passed a null.
-            // We already know next_ptr is not null.
-            let name = unsafe { libbpf_sys::bpf_map__name(next_ptr) };
-            let name = util::c_ptr_to_string(name)?;
-
-            // Get the map def
-            // bpf_map__def can return null but only if it's passed a null.
-            // We already know next_ptr is not null.
-            let def = unsafe { ptr::read(libbpf_sys::bpf_map__def(next_ptr)) };
-
-            // Get the map fd
-            let fd = unsafe { libbpf_sys::bpf_map__fd(next_ptr) };
-            if fd < 0 {
-                return Err(Error::System(-fd));
-            }
-
-            // Add the map to the hashmap
-            obj.maps.insert(
-                name.clone(),
-                Map::new(fd, name, def.type_, def.key_size, def.value_size, next_ptr),
-            );
-            map = next_ptr;
-        }
-
-        // Populate obj.progs
-        let mut prog: *mut libbpf_sys::bpf_program = std::ptr::null_mut();
-        loop {
-            // Get the pointer to the next BPF program
-            let next_ptr = unsafe { libbpf_sys::bpf_object__next_program(obj.ptr, prog) };
-            if next_ptr.is_null() {
-                break;
-            }
-
-            // Get the program name
-            // bpf_program__name never returns NULL, so no need to check the pointer.
-            let name = unsafe { libbpf_sys::bpf_program__name(next_ptr) };
-            let name = util::c_ptr_to_string(name)?;
-
-            // Get the program section
-            // bpf_program__section_name never returns NULL, so no need to check the pointer.
-            let section = unsafe { libbpf_sys::bpf_program__section_name(next_ptr) };
-            let section = util::c_ptr_to_string(section)?;
-
-            // Add the program to the hashmap
-            obj.progs
-                .insert(name.clone(), Program::new(next_ptr, name, section));
-            prog = next_ptr;
-        }
-
-        Ok(obj)
-    }
-
     /// Takes ownership from pointer.
     ///
     /// # Safety
@@ -377,8 +346,59 @@ impl Object {
     /// undefined.
     ///
     /// It is not safe to manipulate `ptr` after this operation.
-    pub unsafe fn from_ptr(ptr: *mut libbpf_sys::bpf_object) -> Result<Self> {
-        Self::new(ptr)
+    pub unsafe fn from_ptr(ptr: NonNull<libbpf_sys::bpf_object>) -> Result<Self> {
+        let mut obj = Object {
+            ptr,
+            maps: HashMap::new(),
+            progs: HashMap::new(),
+        };
+
+        // Populate obj.maps
+        let mut map: *mut libbpf_sys::bpf_map = ptr::null_mut();
+        loop {
+            // Get the pointer to the next BPF map
+            let map_ptr = {
+                let next_ptr = unsafe { libbpf_sys::bpf_object__next_map(obj.ptr.as_ptr(), map) };
+                match NonNull::new(next_ptr) {
+                    Some(map_ptr) => map_ptr,
+                    None => break,
+                }
+            };
+
+            if unsafe { libbpf_sys::bpf_map__autocreate(map_ptr.as_ptr()) } {
+                let map_obj = unsafe { Map::new(map_ptr) }?;
+                obj.maps.insert(map_obj.name().into(), map_obj);
+            }
+
+            map = map_ptr.as_ptr();
+        }
+
+        // Populate obj.progs
+        let mut prog: *mut libbpf_sys::bpf_program = ptr::null_mut();
+        loop {
+            // Get the pointer to the next BPF program
+            let prog_ptr = {
+                let next_ptr =
+                    unsafe { libbpf_sys::bpf_object__next_program(obj.ptr.as_ptr(), prog) };
+                match NonNull::new(next_ptr) {
+                    Some(prog_ptr) => prog_ptr,
+                    None => break,
+                }
+            };
+
+            let program = unsafe { Program::new(prog_ptr) }?;
+
+            // Add the program to the hashmap
+            obj.progs.insert(program.name().into(), program);
+            prog = prog_ptr.as_ptr();
+        }
+
+        Ok(obj)
+    }
+
+    /// Parse the btf information associated with this bpf object.
+    pub fn btf(&self) -> Result<Btf<'_>> {
+        Btf::from_bpf_object(unsafe { &*self.ptr.as_ptr() })
     }
 
     /// Get a reference to `Map` with the name `name`, if one exists.
@@ -426,10 +446,19 @@ impl Object {
     }
 }
 
+impl AsRawLibbpf for Object {
+    type LibbpfType = libbpf_sys::bpf_object;
+
+    /// Retrieve the underlying [`libbpf_sys::bpf_object`].
+    fn as_libbpf_object(&self) -> NonNull<Self::LibbpfType> {
+        self.ptr
+    }
+}
+
 impl Drop for Object {
     fn drop(&mut self) {
         unsafe {
-            libbpf_sys::bpf_object__close(self.ptr);
+            libbpf_sys::bpf_object__close(self.ptr.as_ptr());
         }
     }
 }
