@@ -1,14 +1,20 @@
-use crate::*;
+use crate::libbpf_sys;
 use lazy_static::lazy_static;
-use std::io::{self, Write};
+use std::io;
+use std::io::Write;
+use std::mem;
 use std::os::raw::c_char;
 use std::sync::Mutex;
 
+/// An enum representing the different supported print levels.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy)]
 #[repr(u32)]
 pub enum PrintLevel {
+    /// Print warnings and more severe messages.
     Warn = libbpf_sys::LIBBPF_WARN,
+    /// Print general information and more severe messages.
     Info = libbpf_sys::LIBBPF_INFO,
+    /// Print debug information and more severe messages.
     Debug = libbpf_sys::LIBBPF_DEBUG,
 }
 
@@ -24,6 +30,7 @@ impl From<libbpf_sys::libbpf_print_level> for PrintLevel {
     }
 }
 
+/// The type of callback functions suitable for being provided to [`set_print`].
 pub type PrintCallback = fn(PrintLevel, String);
 
 /// Mimic the default print functionality of libbpf. This way if the user calls `get_print` when no
@@ -56,7 +63,7 @@ extern "C" fn outer_print_cb(
         if level <= min_level {
             let msg = match unsafe { vsprintf::vsprintf(fmtstr, va_list) } {
                 Ok(s) => s,
-                Err(e) => format!("Failed to parse libbpf output: {}", e),
+                Err(e) => format!("Failed to parse libbpf output: {e}"),
             };
             func(level, msg);
         }
@@ -71,14 +78,13 @@ extern "C" fn outer_print_cb(
 /// * `callback` - Either a tuple `(min_level, function)` where `min_level` is the lowest priority
 ///   log message to handle, or `None` to disable all printing.
 ///
-/// This overrides (and is overridden by) [`ObjectBuilder::debug`]
+/// This overrides (and is overridden by) [`ObjectBuilder::debug`][crate::ObjectBuilder::debug]
 ///
 /// # Examples
 ///
 /// To pass all messages to the `log` crate:
 ///
 /// ```
-/// use log;
 /// use libbpf_rs::{PrintLevel, set_print};
 ///
 /// fn print_to_log(level: PrintLevel, msg: String) {
@@ -112,7 +118,7 @@ pub fn set_print(
     mut callback: Option<(PrintLevel, PrintCallback)>,
 ) -> Option<(PrintLevel, PrintCallback)> {
     let real_cb: libbpf_sys::libbpf_print_fn_t = callback.as_ref().and(Some(outer_print_cb));
-    std::mem::swap(&mut callback, &mut *PRINT_CB.lock().unwrap());
+    mem::swap(&mut callback, &mut *PRINT_CB.lock().unwrap());
     unsafe { libbpf_sys::libbpf_set_print(real_cb) };
     callback
 }
@@ -121,7 +127,7 @@ pub fn set_print(
 ///
 /// # Examples
 ///
-/// To temporarliy suppress output:
+/// To temporarily suppress output:
 ///
 /// ```
 /// use libbpf_rs::{get_print, set_print};
