@@ -1,31 +1,48 @@
 use core::ffi::c_void;
-use std::alloc::{alloc_zeroed, dealloc, Layout};
+use std::alloc::alloc_zeroed;
+use std::alloc::dealloc;
+use std::alloc::Layout;
 use std::boxed::Box;
 use std::ffi::CString;
 use std::mem::size_of;
-use std::os::raw::{c_char, c_ulong};
+use std::os::raw::c_char;
+use std::os::raw::c_ulong;
 use std::ptr;
+use std::ptr::NonNull;
 
-use libbpf_sys::{
-    bpf_link, bpf_map, bpf_map_skeleton, bpf_object, bpf_object_skeleton, bpf_prog_skeleton,
-    bpf_program,
-};
+use libbpf_sys::bpf_link;
+use libbpf_sys::bpf_map;
+use libbpf_sys::bpf_map_skeleton;
+use libbpf_sys::bpf_object;
+use libbpf_sys::bpf_object_skeleton;
+use libbpf_sys::bpf_prog_skeleton;
+use libbpf_sys::bpf_program;
 
+use crate::error::IntoError as _;
+use crate::libbpf_sys;
 use crate::util;
-use crate::*;
+use crate::Error;
+use crate::Object;
+use crate::ObjectBuilder;
+use crate::OpenObject;
+use crate::Result;
 
+#[derive(Debug)]
 struct MapSkelConfig {
     name: String,
     p: Box<*mut bpf_map>,
     mmaped: Option<Box<*mut c_void>>,
 }
 
+#[derive(Debug)]
 struct ProgSkelConfig {
     name: String,
     p: Box<*mut bpf_program>,
     link: Box<*mut bpf_link>,
 }
 
+#[allow(missing_docs)]
+#[derive(Debug)]
 pub struct ObjectSkeletonConfigBuilder<'a> {
     data: &'a [u8],
     p: Box<*mut bpf_object>,
@@ -59,6 +76,7 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
         }
     }
 
+    #[allow(missing_docs)]
     pub fn name<T: AsRef<str>>(&mut self, name: T) -> &mut Self {
         self.name = Some(name.as_ref().to_string());
         self
@@ -95,23 +113,23 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
     }
 
     fn build_maps(
-        &mut self,
+        maps: &mut [MapSkelConfig],
         s: &mut bpf_object_skeleton,
         string_pool: &mut Vec<CString>,
     ) -> Option<Layout> {
-        if self.maps.is_empty() {
+        if maps.is_empty() {
             return None;
         }
 
-        s.map_cnt = self.maps.len() as i32;
+        s.map_cnt = maps.len() as i32;
         s.map_skel_sz = size_of::<bpf_map_skeleton>() as i32;
 
-        let layout = Layout::array::<bpf_map_skeleton>(self.maps.len())
+        let layout = Layout::array::<bpf_map_skeleton>(maps.len())
             .expect("Failed to allocate memory for maps skeleton");
 
         unsafe {
             s.maps = alloc_zeroed(layout) as *mut bpf_map_skeleton;
-            for (i, map) in self.maps.iter_mut().enumerate() {
+            for (i, map) in maps.iter_mut().enumerate() {
                 let current_map = s.maps.add(i);
 
                 // Opt to panic on error here. We've already allocated memory and we'd rather not
@@ -131,23 +149,23 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
     }
 
     fn build_progs(
-        &mut self,
+        progs: &mut [ProgSkelConfig],
         s: &mut bpf_object_skeleton,
         string_pool: &mut Vec<CString>,
     ) -> Option<Layout> {
-        if self.progs.is_empty() {
+        if progs.is_empty() {
             return None;
         }
 
-        s.prog_cnt = self.progs.len() as i32;
+        s.prog_cnt = progs.len() as i32;
         s.prog_skel_sz = size_of::<bpf_prog_skeleton>() as i32;
 
-        let layout = Layout::array::<bpf_prog_skeleton>(self.progs.len())
+        let layout = Layout::array::<bpf_prog_skeleton>(progs.len())
             .expect("Failed to allocate memory for progs skeleton");
 
         unsafe {
             s.progs = alloc_zeroed(layout) as *mut bpf_prog_skeleton;
-            for (i, prog) in self.progs.iter_mut().enumerate() {
+            for (i, prog) in progs.iter_mut().enumerate() {
                 let current_prog = s.progs.add(i);
 
                 // See above for `expect()` rationale
@@ -161,6 +179,7 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
         Some(layout)
     }
 
+    #[allow(missing_docs)]
     pub fn build(mut self) -> Result<ObjectSkeletonConfig<'a>> {
         // Holds `CString`s alive so pointers to them stay valid
         let mut string_pool = Vec::new();
@@ -178,14 +197,14 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
         s.data = self.data.as_ptr() as *mut c_void;
         s.data_sz = self.data.len() as c_ulong;
 
-        s.obj = &mut *self.p;
+        // Give s ownership over the box
+        s.obj = Box::into_raw(self.p);
 
-        let maps_layout = self.build_maps(&mut s, &mut string_pool);
-        let progs_layout = self.build_progs(&mut s, &mut string_pool);
+        let maps_layout = Self::build_maps(&mut self.maps, &mut s, &mut string_pool);
+        let progs_layout = Self::build_progs(&mut self.progs, &mut s, &mut string_pool);
 
         Ok(ObjectSkeletonConfig {
             inner: s,
-            obj: self.p,
             maps: self.maps,
             progs: self.progs,
             maps_layout,
@@ -203,9 +222,9 @@ impl<'a> ObjectSkeletonConfigBuilder<'a> {
 /// * free any allocated memory on drop
 ///
 /// This struct can be moved around at will. Upon drop, all allocated resources will be freed
+#[derive(Debug)]
 pub struct ObjectSkeletonConfig<'a> {
     inner: bpf_object_skeleton,
-    obj: Box<*mut bpf_object>,
     maps: Vec<MapSkelConfig>,
     progs: Vec<ProgSkelConfig>,
     /// Layout necessary to `dealloc` memory
@@ -218,14 +237,25 @@ pub struct ObjectSkeletonConfig<'a> {
     _string_pool: Vec<CString>,
 }
 
-impl<'a> ObjectSkeletonConfig<'a> {
+impl ObjectSkeletonConfig<'_> {
+    #[allow(missing_docs)]
     pub fn get(&mut self) -> &mut bpf_object_skeleton {
         &mut self.inner
     }
 
     /// Warning: the returned pointer is only valid while the `ObjectSkeletonConfig` is alive.
-    pub fn object_ptr(&mut self) -> *mut bpf_object {
-        *self.obj
+    ///
+    /// # Panic
+    /// This method panics if the inner [`bpf_object_skeleton`] has not be initialized.
+    ///
+    /// To initialize it, first call [`Self::get`] and initialize the skeleton.
+    pub fn object_ptr(&mut self) -> NonNull<bpf_object> {
+        NonNull::new(unsafe { *self.inner.obj }).expect(
+            r#"
+        The generated code failed to initialize bpf_object_skeleton.obj pointer through the use
+        of `bpf_object__open_skeleton(skel_config.get(), &open_opts)`
+        "#,
+        )
     }
 
     /// Returns the `mmaped` pointer for a map at the specified `index`.
@@ -234,15 +264,28 @@ impl<'a> ObjectSkeletonConfig<'a> {
     /// `ObjectSkeletonConfigBuilder::map`. Index starts at 0.
     ///
     /// Warning: the returned pointer is only valid while the `ObjectSkeletonConfig` is alive.
-    pub fn map_mmap_ptr(&mut self, index: usize) -> Result<*mut c_void> {
+    pub fn map_mmap_ptr(&self, index: usize) -> Result<*const c_void> {
         if index >= self.maps.len() {
-            return Err(Error::Internal(format!("Invalid map index: {}", index)));
+            return Err(Error::with_invalid_data(format!(
+                "Invalid map index: {index}"
+            )));
         }
 
-        self.maps[index].mmaped.as_ref().map_or_else(
-            || Err(Error::Internal("Map does not have mmaped ptr".to_string())),
-            |p| Ok(**p),
-        )
+        let p = self.maps[index]
+            .mmaped
+            .as_ref()
+            .ok_or_invalid_data(|| "Map does not have mmaped ptr")?;
+        Ok(**p)
+    }
+
+    /// Returns the `mmaped` pointer for a map at the specified `index`.
+    ///
+    /// The index is determined by the order in which the map was passed to
+    /// `ObjectSkeletonConfigBuilder::map`. Index starts at 0.
+    ///
+    /// Warning: the returned pointer is only valid while the `ObjectSkeletonConfig` is alive.
+    pub fn map_mmap_ptr_mut(&mut self, index: usize) -> Result<*mut c_void> {
+        self.map_mmap_ptr(index).map(|p| p.cast_mut())
     }
 
     /// Returns the link pointer for a prog at the specified `index`.
@@ -253,14 +296,16 @@ impl<'a> ObjectSkeletonConfig<'a> {
     /// Warning: the returned pointer is only valid while the `ObjectSkeletonConfig` is alive.
     pub fn prog_link_ptr(&mut self, index: usize) -> Result<*mut bpf_link> {
         if index >= self.progs.len() {
-            return Err(Error::Internal(format!("Invalid prog index: {}", index)));
+            return Err(Error::with_invalid_data(format!(
+                "Invalid prog index: {index}"
+            )));
         }
 
         Ok(*self.progs[index].link)
     }
 }
 
-impl<'a> Drop for ObjectSkeletonConfig<'a> {
+impl Drop for ObjectSkeletonConfig<'_> {
     // Note we do *not* run `libbpf_sys::bpf_object__destroy_skeleton` here.
     //
     // Couple reasons:
@@ -287,5 +332,83 @@ impl<'a> Drop for ObjectSkeletonConfig<'a> {
                 dealloc(self.inner.progs as _, layout);
             }
         }
+
+        let _ = unsafe { Box::from_raw(self.inner.obj) };
     }
 }
+
+/// A trait for skeleton builder.
+pub trait SkelBuilder<'a> {
+    /// Define that when BPF object is opened, the returned type should implement the [`OpenSkel`] trait
+    type Output: OpenSkel;
+
+    /// Open eBPF object and return [`OpenSkel`]
+    fn open(self) -> Result<Self::Output>;
+
+    /// Open eBPF object with [`libbpf_sys::bpf_object_open_opts`] and return [`OpenSkel`]
+    fn open_opts(self, open_opts: libbpf_sys::bpf_object_open_opts) -> Result<Self::Output>;
+
+    /// Get a reference to [`ObjectBuilder`]
+    fn object_builder(&self) -> &ObjectBuilder;
+
+    /// Get a mutable reference to [`ObjectBuilder`]
+    fn object_builder_mut(&mut self) -> &mut ObjectBuilder;
+}
+
+/// A trait for opened skeleton.
+///
+/// In addition to the methods defined in this trait, skeletons that implement this trait will also
+/// have bespoke implementations of a few additional methods to facilitate access to global
+/// variables of the BPF program. These methods will be named `bss()`, `data()`, and `rodata()`.
+/// Each corresponds to the variables stored in the BPF ELF program section of the same name.
+/// However if your BPF program lacks one of these sections the corresponding rust method will not
+/// be generated.
+///
+/// The type of the value returned by each of these methods will be specific to your BPF program.
+/// A common convention is to define a single global variable in the BPF program with a struct type
+/// containing a field for each configuration parameter <sup>\[[source]\]</sup>.  libbpf-rs
+/// auto-generates this pattern for you without you having to define such a struct type in your BPF
+/// program. It does this by examining each of the global variables in your BPF program's `.bss`,
+/// `.data`, and `.rodata` sections and then creating rust struct types `<yourprogram>_bss_types`,
+/// `<yourprogram>_data_types`, and `<yourprogram>_rodata_types`. Since these struct types are
+/// specific to the layout of your BPF program, they are not documented in this crate. However you
+/// can see documentation for them by running `cargo doc` in your own project and looking at the
+/// `imp` module. You can also view their implementation by looking at the generated skeleton rust
+/// source file. The use of these methods can also be seen in the examples 'capable', 'runqslower',
+/// and 'tproxy'.
+///
+/// If you ever doubt whether libbpf-rs has placed a particular variable in the correct struct
+/// type, you can see which section each global variable is stored in by examing the output of the
+/// following command (after a successful build):
+///
+/// ```sh
+/// bpf-objdump --syms ./target/bpf/*.bpf.o
+/// ```
+///
+/// [source]: https://nakryiko.com/posts/bcc-to-libbpf-howto-guide/#application-configuration
+pub trait OpenSkel {
+    /// Define that when BPF object is loaded, the returned type should implement the [`Skel`] trait
+    type Output: Skel;
+
+    /// Load BPF object and return [`Skel`].
+    fn load(self) -> Result<Self::Output>;
+
+    /// Get a reference to [`OpenObject`].
+    fn open_object(&self) -> &OpenObject;
+
+    /// Get a mutable reference to [`OpenObject`].
+    fn open_object_mut(&mut self) -> &mut OpenObject;
+}
+
+/// A trait for loaded skeleton.
+pub trait Skel {
+    /// Attach BPF object.
+    fn attach(&mut self) -> Result<()> {
+        unimplemented!()
+    }
+    /// Get a reference to [`Object`].
+    fn object(&self) -> &Object;
+
+    /// Get a mutable reference to [`Object`].
+    fn object_mut(&mut self) -> &mut Object;
+}
